import { __assign, __decorate, __param, __metadata } from 'tslib';
import { InjectionToken, Inject, ɵɵdefineInjectable, ɵɵinject, Injectable, Input, Component, NgModule } from '@angular/core';
import { Subject, share, Observable } from 'rxjs';
import { EventManager } from '@angular/platform-browser';
import { DOCUMENT, CommonModule } from '@angular/common';

var HOTKEY_OPTIONS = new InjectionToken('HotkeyOptions');

var _defaultOptions = {
    disableCheatSheet: false,
    cheatSheetTitle: 'Keyboard Shortcuts:',
    cheatSheetHotkey: 'shift.?',
    cheatSheetHotkeyDescription: 'Show / hide this help menu',
    cheatSheetCloseEsc: false,
    cheatSheetCloseEscDescription: 'Hide this help menu'
};
var NgxHotkeysService = /** @class */ (function () {
    function NgxHotkeysService(eventManager, config, doc) {
        var _this = this;
        this.eventManager = eventManager;
        this.config = config;
        this.registeredHotkeys = new Set();
        this.pausedHotkeys = new Set();
        this._cheatSheetToggled = new Subject();
        this.bindingSubscriptions = new Map();
        this.document = doc;
        this.defaults = { element: this.document };
        this.configuration = __assign(__assign({}, _defaultOptions), this.config);
        if (!this.configuration.disableCheatSheet) {
            this.register({
                combo: this.configuration.cheatSheetHotkey,
                handler: function () {
                    _this._cheatSheetToggled.next(true);
                },
                description: this.configuration.cheatSheetHotkeyDescription
            });
        }
        if (this.configuration.cheatSheetCloseEsc) {
            this.register({
                combo: 'esc',
                handler: function () {
                    _this._cheatSheetToggled.next(false);
                },
                description: this.configuration.cheatSheetCloseEscDescription
            });
        }
    }
    Object.defineProperty(NgxHotkeysService.prototype, "hotkeys", {
        /**
         * Returns the registered hotkeys as array.
         * @returns all registered hotkeys
         */
        get: function () {
            return Array.from(this.registeredHotkeys);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgxHotkeysService.prototype, "cheatSheetToggled", {
        /**
         * Returns an Observable stream indicating the cheatsheets visibility was toggled.
         * @returns stream indicating the cheatsheets visibility was toggled
         */
        get: function () {
            return this._cheatSheetToggled.asObservable().pipe(share());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgxHotkeysService.prototype, "options", {
        /**
         * Returns the options used to configure this service.
         */
        get: function () {
            return this.configuration;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Registers a new hotkey/new hotkeys with it's/their handler(s).
     * @param hotkey hotkeys to listen for
     * @param unpausing flag indicating if the hotkeys should be unpaused
     */
    NgxHotkeysService.prototype.register = function (hotkey, unpausing) {
        var _this = this;
        if (unpausing === void 0) { unpausing = false; }
        var hotkeys = [].concat(hotkey);
        if (unpausing) {
            hotkeys = Array.from(this.pausedHotkeys);
        }
        hotkeys.forEach(function (h) {
            if (unpausing) {
                _this.pausedHotkeys.delete(h);
            }
            _this.registeredHotkeys.add(h);
            _this.bindingSubscriptions.set(h.combo, _this.bindToEventManager(h)
                .subscribe(function () {
                h.handler();
            }));
        });
    };
    /**
     * Removes a/the registered hotkey(s).
     * @param hotkey hotkey filter
     * @param pausing flag indicating if the hotkeys should be paused
     */
    NgxHotkeysService.prototype.unregister = function (hotkey, pausing) {
        var _this = this;
        if (pausing === void 0) { pausing = false; }
        var hotkeys = [].concat(hotkey);
        hotkeys.forEach(function (h) {
            _this.registeredHotkeys.delete(h);
            if (pausing) {
                _this.pausedHotkeys.add(h);
            }
            _this.unbindFromEventManager(h);
        });
    };
    /**
     * Returns all hotkeys matching the passed combo(s).
     * @param combo combo to match against
     * @returns all matched hotkeys
     */
    NgxHotkeysService.prototype.get = function (combo) {
        return this.hotkeys.filter(function (h) { return h.combo === combo; });
    };
    /**
     * Stops listening for the specified hotkeys.
     * @param hotkey hotkey filter
     */
    NgxHotkeysService.prototype.pause = function (hotkey) {
        if (!hotkey) {
            return this.pause(this.hotkeys);
        }
        var hotkeys = [].concat(hotkey);
        this.unregister(hotkeys, true);
    };
    /**
     * Resumes listening for the specified hotkeys.
     * @param hotkey hotkey filter
     */
    NgxHotkeysService.prototype.unpause = function (hotkey) {
        if (!hotkey) {
            return this.unpause(this.hotkeys);
        }
        var hotkeys = [].concat(hotkey);
        this.register(hotkeys, true);
    };
    /**
     * Resets all hotkeys.
     */
    NgxHotkeysService.prototype.reset = function () {
        var _this = this;
        this.registeredHotkeys.forEach(function (h) { return _this.unbindFromEventManager(h); });
        this.registeredHotkeys.clear();
        this.pausedHotkeys.clear();
    };
    NgxHotkeysService.prototype.ngOnDestroy = function () {
        this.reset();
    };
    NgxHotkeysService.prototype.bindToEventManager = function (hotkey) {
        var _this = this;
        var merged = __assign(__assign({}, this.defaults), hotkey);
        var event = "keydown." + merged.combo.replace('+', '.');
        return new Observable(function (observer) {
            var handler = function (e) {
                e.preventDefault();
                observer.next(e);
            };
            var dispose = _this.eventManager.addEventListener(merged.element, event, handler);
            return function () {
                dispose();
            };
        });
    };
    NgxHotkeysService.prototype.unbindFromEventManager = function (hotkey) {
        var merged = __assign(__assign({}, this.defaults), hotkey);
        var keySub = this.bindingSubscriptions.get(merged.combo);
        if (keySub && !keySub.closed) {
            keySub.unsubscribe();
            this.bindingSubscriptions.delete(merged.combo);
        }
    };
    NgxHotkeysService.ctorParameters = function () { return [
        { type: EventManager },
        { type: undefined, decorators: [{ type: Inject, args: [HOTKEY_OPTIONS,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
    ]; };
    NgxHotkeysService.ɵprov = ɵɵdefineInjectable({ factory: function NgxHotkeysService_Factory() { return new NgxHotkeysService(ɵɵinject(EventManager), ɵɵinject(HOTKEY_OPTIONS), ɵɵinject(DOCUMENT)); }, token: NgxHotkeysService, providedIn: "root" });
    NgxHotkeysService = __decorate([
        Injectable({
            providedIn: 'root'
        }),
        __param(1, Inject(HOTKEY_OPTIONS)),
        __param(2, Inject(DOCUMENT)),
        __metadata("design:paramtypes", [EventManager, Object, Object])
    ], NgxHotkeysService);
    return NgxHotkeysService;
}());

var NgxCheatsheetComponent = /** @class */ (function () {
    function NgxCheatsheetComponent(hotkeysService) {
        this.hotkeysService = hotkeysService;
        this.helpVisible = false;
        this._map = {
            meta: '\u2318',
            shift: '\u21E7',
            left: '\u2190',
            right: '\u2192',
            up: '\u2191',
            down: '\u2193',
            'return': '\u23CE',
            backspace: '\u232B' // ⌫
        };
        this.title = this.hotkeysService.options.cheatSheetTitle || this.title;
    }
    NgxCheatsheetComponent.prototype.ngOnInit = function () {
        var _this = this;
        this._subscription = this.hotkeysService.cheatSheetToggled
            .subscribe(function (isOpen) {
            if (isOpen !== false) {
                _this.hotkeys = _this.hotkeysService.hotkeys
                    .filter(function (hotkey) { return hotkey.description; });
            }
            if (isOpen === false) {
                _this.helpVisible = false;
            }
            else {
                _this.toggleCheatSheet();
            }
        });
    };
    NgxCheatsheetComponent.prototype.formatHotkey = function (hotkey) {
        var combo = Array.isArray(hotkey.combo) ? hotkey.combo[0] : hotkey.combo;
        var sequence = combo.split(/[\s]/);
        for (var i = 0; i < sequence.length; i++) {
            sequence[i] = this.symbolize(sequence[i]);
        }
        return sequence;
    };
    NgxCheatsheetComponent.prototype.symbolize = function (combo) {
        var comboSplit = combo.split('.');
        for (var i = 0; i < comboSplit.length; i++) {
            // try to resolve command / ctrl based on OS:
            if (comboSplit[i] === 'mod') {
                if (window.navigator && window.navigator.platform.indexOf('Mac') >= 0) {
                    comboSplit[i] = 'meta';
                }
                else {
                    comboSplit[i] = 'ctrl';
                }
            }
            comboSplit[i] = this._map[comboSplit[i]] || comboSplit[i];
        }
        return comboSplit.join(' + ');
    };
    NgxCheatsheetComponent.prototype.ngOnDestroy = function () {
        if (this._subscription && !this._subscription.closed) {
            this._subscription.unsubscribe();
        }
    };
    NgxCheatsheetComponent.prototype.toggleCheatSheet = function () {
        this.helpVisible = !this.helpVisible;
    };
    NgxCheatsheetComponent.ctorParameters = function () { return [
        { type: NgxHotkeysService }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], NgxCheatsheetComponent.prototype, "title", void 0);
    NgxCheatsheetComponent = __decorate([
        Component({
            selector: 'ngx-cheatsheet',
            template: "<div class=\"ngx-cheatsheet fade\" [ngClass]=\"{'in': helpVisible}\" style=\"display:none\">\n  <div class=\"ngx-cheatsheet__hotkeys\">\n    <h4 class=\"ngx-cheatsheet__hotkeys__title\">{{ title }}</h4>\n    <table>\n      <tbody>\n        <tr *ngFor=\"let hotkey of hotkeys\">\n          <td class=\"ngx-cheatsheet__hotkeys__keys\">\n            <span class=\"ngx-cheatsheet__hotkeys__keys__key\">{{ formatHotkey(hotkey) }}</span>\n          </td>\n          <td class=\"ngx-cheatsheet__hotkeys__keys__text\">{{ hotkey.description }}</td>\n        </tr>\n      </tbody>\n    </table>\n    <div class=\"ngx-cheatsheet__hotkeys__close\" (click)=\"toggleCheatSheet()\">&#215;</div>\n  </div>\n</div>",
            styles: [".ngx-cheatsheet{display:table!important;position:fixed;width:100%;height:100%;top:0;left:0;color:#333;font-size:1em;background-color:rgba(255,255,255,.9)}.ngx-cheatsheet.fade{z-index:-1024;visibility:hidden;opacity:0;transition:opacity .15s linear}.ngx-cheatsheet.fade.in{z-index:10002;visibility:visible;opacity:1}.ngx-cheatsheet__hotkeys{width:100%;height:100%;display:table-cell;vertical-align:middle}.ngx-cheatsheet__hotkeys__title{font-weight:700;text-align:center;font-size:1.2em}.ngx-cheatsheet__hotkeys table{margin:auto;color:#333}.ngx-cheatsheet__hotkeys__keys{padding:5px;text-align:right}.ngx-cheatsheet__hotkeys__keys__key{display:inline-block;color:#fff;background-color:#333;border:1px solid #333;border-radius:5px;text-align:center;margin-right:5px;box-shadow:inset 0 1px 0 #666,0 1px 0 #bbb;padding:5px 9px;font-size:1em}.ngx-cheatsheet__hotkeys__keys__text{padding-left:10px;font-size:1em}.ngx-cheatsheet__hotkeys__close{position:fixed;top:20px;right:20px;font-size:2em;font-weight:700;padding:5px 10px;border:1px solid #ddd;border-radius:5px;min-height:45px;min-width:45px;text-align:center;cursor:pointer}@media all and (max-width:500px){.ngx-cheatsheet__hotkeys{font-size:.8em}}@media all and (min-width:750px){.ngx-cheatsheet__hotkeys{font-size:1.2em}}"]
        }),
        __metadata("design:paramtypes", [NgxHotkeysService])
    ], NgxCheatsheetComponent);
    return NgxCheatsheetComponent;
}());

var NgxHotkeysModule = /** @class */ (function () {
    function NgxHotkeysModule() {
    }
    NgxHotkeysModule_1 = NgxHotkeysModule;
    NgxHotkeysModule.forRoot = function (options) {
        return {
            ngModule: NgxHotkeysModule_1,
            providers: [
                NgxHotkeysService,
                {
                    provide: HOTKEY_OPTIONS,
                    useValue: options
                }
            ]
        };
    };
    var NgxHotkeysModule_1;
    NgxHotkeysModule = NgxHotkeysModule_1 = __decorate([
        NgModule({
            imports: [
                CommonModule
            ],
            declarations: [
                NgxCheatsheetComponent
            ],
            exports: [
                NgxCheatsheetComponent
            ]
        })
    ], NgxHotkeysModule);
    return NgxHotkeysModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { NgxCheatsheetComponent, NgxHotkeysModule, NgxHotkeysService, HOTKEY_OPTIONS as ɵa };
//# sourceMappingURL=ngx-hotkeys-vnc.js.map
